@@ -1,8 +1,10 @@
 import glob
+import math
 import os
 import shutil
 import sys
 import termios
+import threading
 
 import cuitools.subp
 
@@ -53,6 +55,148 @@ def Key():
     return ch
 
 
+def Inputfilegui(title, path=None):
+    terminal_size = shutil.get_terminal_size()
+    # path = os.path.expanduser("/usr/share")
+    if path is None:
+        path = os.path.expanduser("~")
+    filelist = glob.glob(path + "/*")
+    printdata = list(map(lambda n: os.path.basename(n) + "/" * os.path.isdir(n), filelist))
+    lenfilelist = max(list(map(lambda n: subp.width_kana(n), printdata))) + 1
+    print(filelist)
+    print("\033[1;1H\033[2J┏" + "━" * (terminal_size[0] - 2) + "┓")
+    print("\033[3;1H┣" + "━" * (terminal_size[0] - 2) + "┫")
+
+    row = math.floor((terminal_size[0] - 2) / lenfilelist)
+    # print(len(filelist),row,terminal_size[0]-2,lenfilelist)
+    k = ""
+    page = 0
+    select = page * row * (terminal_size[1] - 5) + 0
+    event = threading.Event()
+    th = threading.Thread(target=subp.th1, args=([filelist, title, path, event]))
+    th.start()
+    try:
+        while k != "\n":
+            print("\033[" + str(terminal_size[1]) + ";1H┗" + "━" * (terminal_size[0] - 2) + "┛", end="")
+            print("\033[4;1H", end="")
+            if lenfilelist != -1:
+                for i in range(math.ceil(len(printdata) / row)):
+                    if i > terminal_size[1] - 6:
+                        break
+                    print("\033[2K┃", end="")
+                    tmp = 0
+                    for j in range(row):
+                        if len(printdata) - 1 < page * (row * (terminal_size[1] - 5)) + (i * row + j):
+                            break
+                        tmp += 1
+                        if select == page * row * (terminal_size[1] - 5) + (i * row + j):
+                            print(
+                                "\033[7m" + subp.isdir(
+                                    subp.ljust_kana(printdata[page * row * (terminal_size[1] - 5) + (i * row + j)],
+                                                    lenfilelist), True) + "\033[0m", end="")
+                        else:
+                            print(subp.isdir(
+                                subp.ljust_kana(printdata[page * row * (terminal_size[1] - 5) + (i * row + j)],
+                                                lenfilelist)), end="")
+                    print(" " * ((terminal_size[0] - 2) - lenfilelist * tmp) + "┃")
+
+                print(("┃" + " " * (terminal_size[0] - 2) + "┃\n") * (
+                        terminal_size[1] - (math.ceil(len(printdata) / row) + 5)),
+                      end="")
+                print("\033[2K┃" + subp.center_kana(
+                    "page " + str(page + 1) + "/" + str(math.ceil(len(printdata) / (row * (terminal_size[1] - 5)))),
+                    terminal_size[0] - 2) + "┃")
+            else:
+                print("┃" + subp.center_kana("No file or directory", terminal_size[0] - 2) + "┃")
+                print(("┃" + " " * (terminal_size[0] - 2) + "┃\n") * (terminal_size[1] - 5), end="")
+
+            k = Key()
+            if k == "\x1b":
+                k = Key()
+                if k == "[":
+                    k = Key()
+                    if k == "A":
+                        if select - row > -1:
+                            select -= row
+                            if select < page * row * (terminal_size[1] - 5):
+                                page -= 1
+                    if k == "B":
+                        if select + row < len(filelist):
+                            select += row
+                            if select >= (page + 1) * row * (terminal_size[1] - 5):
+                                page += 1
+                    if k == "C":
+                        if select + 1 < len(filelist):
+                            select += 1
+                            if select >= (page + 1) * row * (terminal_size[1] - 5):
+                                page += 1
+                    if k == "D":
+                        if select - 1 > -1:
+                            select -= 1
+                            if select < page * row * (terminal_size[1] - 5):
+                                page -= 1
+                    if k == "1":
+                        k = Key()
+                        if k == "5":
+                            Key()
+                            event.set()
+                            filelist = glob.glob(path + "/*")
+                            if len(filelist) != 0:
+                                printdata = list(map(lambda n: os.path.basename(n) + "/" * os.path.isdir(n), filelist))
+                                lenfilelist = max(list(map(lambda n: subp.width_kana(n), printdata))) + 1
+                                row = math.floor((terminal_size[0] - 2) / lenfilelist)
+                            else:
+                                lenfilelist = -1
+                            if len(filelist) - 1 < select:
+                                page = 0
+                                select = page * row * (terminal_size[1] - 5) + 0
+                            event.clear()
+                            th = threading.Thread(target=subp.th1, args=([filelist, title, path, event, True]))
+                            th.start()
+
+            elif k == "\n":
+                if os.path.isdir(filelist[select]):
+                    k = ""
+                    event.set()
+                    path = filelist[select]
+                    page = 0
+                    select = page * row * (terminal_size[1] - 5) + 0
+                    filelist = glob.glob(path + "/*")
+                    if len(filelist) != 0:
+                        printdata = list(map(lambda n: os.path.basename(n) + "/" * os.path.isdir(n), filelist))
+                        lenfilelist = max(list(map(lambda n: subp.width_kana(n), printdata))) + 1
+                        row = math.floor((terminal_size[0] - 2) / lenfilelist)
+                    else:
+                        lenfilelist = -1
+                    event.clear()
+                    th = threading.Thread(target=subp.th1, args=([filelist, title, path, event]))
+                    th.start()
+                elif lenfilelist == -1:
+                    k = ""
+            elif k == "\x7f":
+                event.set()
+                path = "/".join(path.split("/")[:-1])
+                page = 0
+                select = page * row * (terminal_size[1] - 5) + 0
+                filelist = glob.glob(path + "/*")
+                if len(filelist) != 0:
+                    printdata = list(map(lambda n: os.path.basename(n) + "/" * os.path.isdir(n), filelist))
+                    lenfilelist = max(list(map(lambda n: subp.width_kana(n), printdata))) + 1
+                    row = math.floor((terminal_size[0] - 2) / lenfilelist)
+                else:
+                    lenfilelist = -1
+                event.clear()
+                th = threading.Thread(target=subp.th1, args=([filelist, title, path, event]))
+                th.start()
+    except KeyboardInterrupt:
+        pass
+    else:
+        reset()
+        event.set()
+        return filelist[select]
+    event.set()
+
+
 def Inputfile(text, textcolor="\033[38;5;10m"):
     k = ""
     pk = ""
@@ -97,7 +241,7 @@ def Inputfile(text, textcolor="\033[38;5;10m"):
                 elif j != 1:
                     pk = tfl[0][0:len(pk) + j]
 
-        print("\033[" + str(int(terminal_size[1] / 2)) + ";1H" + "-" * terminal_size[0])
+        print("\033[" + str(int(terminal_size[1] / 2)) + ";1H" + "━" * terminal_size[0])
         if len(fl) == 0:
             fl.append("empty")
         fll = []
@@ -285,22 +429,23 @@ def printlist(title, listdata=None, center=True):
                                 " "))
         k = Key()
         if k == "\x1b":
-            Key()
             k = Key()
-            if k == "A":
-                if select != 0:
-                    if select - page != 0:
-                        select -= 1
-                    else:
-                        page -= 1
-                        select -= 1
-            elif k == "B":
-                if len(listdata) - 1 != select:
-                    if select - page + 2 != terminal_size[1] - 2:
-                        select += 1
-                    else:
-                        page += 1
-                        select += 1
+            if k == "[":
+                k = Key()
+                if k == "A":
+                    if select != 0:
+                        if select - page != 0:
+                            select -= 1
+                        else:
+                            page -= 1
+                            select -= 1
+                elif k == "B":
+                    if len(listdata) - 1 != select:
+                        if select - page + 2 != terminal_size[1] - 2:
+                            select += 1
+                        else:
+                            page += 1
+                            select += 1
     reset()
     if empty == 1:
         return -1
